#!/usr/bin/perl -w
#Скрипт для застройки указанной планеты максимальным количеством
#добывалок указанного типа; также для оптимизации
#расстановки наместников при помощи программы DCGov
use strict;
use POSIX;
use LWP::UserAgent;
use HTTP::Cookies;
use Compress::Zlib;
$| = 1;

my $login = shift;
my $pass = shift;

our $url_base='.the-game.ru';
our $cookies=HTTP::Cookies->new('file'=>'./cookies.lwp','autosave'=>0);
my $browser=LWP::UserAgent->new('agent' => 'Opera/9.10 (X11; Linux i686; U; en)',
								  'cookie_jar' => $cookies,
								  'requests_redirectable' => ['GET', 'POST']);
$browser->default_header('Accept' => 'text/html, application/xml;q=0.9, application/xhtml+xml, image/png, image/jpeg, image/gif, image/x-xbitmap, */*;q=0.1',
						'Accept-Charset' => 'iso-8859-1, utf-8, utf-16, *;q=0.1',
						'Accept-Language' => 'ru,en;q=0.9',
						'Accept-Encoding' => 'deflate, gzip, x-gzip, identity, *;q=0',
						);

############# MAIN PROGRAMM #############
my ($AccType, $source) = &Login;

print "Запустить dcgov для расчета коррупции после завершения работы скрипта? [Y/n]";
my $build_ID = <STDIN>;
chomp ($build_ID);

&GovsOptimiser($AccType, $source);
&Logoff($AccType);

############# PROCEDURES #############
sub GovsOptimiser($){
 my ($AccType, $source) = @_;
 my $output_planets = $login.'_planets.txt';
 my $output_govs = $login.'_govs.txt';
 
 my @my_govs;
 my @my_planets;
  
#Getting HW coords
 print STDOUT "\r\nUncompress overview page data to get HW coords...\r\n";
 my ($hw_x,$hw_y);
 if ($source=~m#<a href="/planet/\?planetid=(\d+):(\d+)&amp;p=[^"]*" class="TAB_MENU">#i){
   push (@my_planets,{('x'=>$1, 'y'=>$2, 'income'=>0, 'max_income'=>0, 'corrupt'=>0)});
   ($hw_x,$hw_y)=($1,$2);
 }
 
# Get Planets list
 print STDOUT "\r\nGet overview planets frame...\r\n";
 my $response=$browser->get("http://$AccType".$url_base.'/frames/userplanets/on/overview/');
 print STDOUT "\r\nUncompress overview planets frame data...\r\n";
 $source=Compress::Zlib::memGunzip(\($response->content)[0]);
 while ($source=~/\?planetid=(\d+):(\d+)\"\starget/g){
    push (@my_planets,{('x'=>$1, 'y'=>$2, 'income'=>0, 'full_income'=>0, 'corrupt'=>0)}) if (($1!=$hw_x) and ($2!=$hw_y));
 }
 
# get statistics foreach planet
 foreach my $el (@my_planets){
    $response=$browser->get("http://$AccType".$url_base.'/frames/planetinfo/on/planet/\?planetid='.$el->{'x'}.':'.$el->{'y'});
    $source=Compress::Zlib::memGunzip(\($response->content)[0]);
    if ($source =~ m/<td class=\"sim_table\" align=\"left\">Кредиты<\/td><td class=\"sim_table\" align=\"right\">(\d+)<\/td>/){
        $el->{'income'}=$1;
    }
    if ($source =~ m/<td colspan=\"2\" class="hot\d+">Коррупция: (\d+)\%/){
        $el->{'corrupt'}=$1;
    }
    $el->{'full_income'} = floor(100.0 * $el->{'income'} / (100.0 - $el->{'corrupt'}) + 0.5);
    
    printf STDOUT "Planet %d:%d Income= %d, Corruption= %d%% -> Full Income= %d\r\n", $el->{'x'}, $el->{'y'}, $el->{'income'}, $el-> {'corrupt'}, $el->{'full_income'};
 }
 
# get governeurs locations
foreach my $el (@my_planets){
    $response=$browser->get("http://$AccType".$url_base.'/frames/planet_fleets/on/planet/\?planetid='.$el->{'x'}.':'.$el->{'y'});
    $source=Compress::Zlib::memGunzip(\($response->content)[0]);
    if ($source=~m/buildings\/13.gif/){
        my $cnt=1;
        my $tail=$';
        if ($tail=~m/\.?Наместник <small>\(x(\d+)\)<\/small>/){
            $cnt=$1;
        }
        for (1..$cnt){
            push (@my_govs,{('x'=>$el->{'x'},'y'=>$el->{'y'})});
        }
        print STDOUT $el->{'x'}.':'.$el->{'y'}." -- ".$cnt." governour(s) found!\r\n";
    }

 
#DCGov income files creating
 open (FILE,">$output_planets");
 for my $el (@my_planets){
    printf FILE "%d:%d;%d;%d\n", $el->{'x'}, $el->{'y'}, $el->{'full_income'}, -1;
 }
 close FILE;

 open (FILE,">$output_govs");
 for my $el (@my_govs){
    printf FILE "%d:%d\n", $el->{'x'}, $el->{'y'};
 }
 close FILE;

if ($build_ID =~ /^y|yes$/i) {
 #print STDOUT `dcgov -e $output_planets $output_govs 100`;
  #open (LOG,">$logfile");
 #print LOG `dcgov -e $output_planets $output_govs 100`;
  #close(LOG);
   tee (`./dcgov -e $output_planets $output_govs 100`);
 unlink($output_planets,$output_govs);
}
 print STDOUT "\nOK\n";
}
}

sub tee {
   my $logfile = $login.'_DCGov_LOG.txt';
   print @_;

   open (LOG,">$logfile");
       print LOG @_;
   close(LOG);
}

sub Login(){
# login
 my $source;
 my $response=$browser->post('http://www.the-game.ru',
    [
     'action'=>'login',
     'login'=>$login,
     'pwd'=>$pass,
    ]);
	$source=Compress::Zlib::memGunzip(\($response->content)[0]);
	$pass=~s/#/%23/;
	$response=$browser->get('http://www'.$url_base."/overview/?&action=login&login=$login&pwd=$pass") if $source=~/<input/;
	$source=Compress::Zlib::memGunzip(\($response->content)[0]);
	do{print STDOUT "\r\n!WARNING! $login: invalid user name and/or password. Please check this manually.\r\n";next;}
		if $source=~/<input/;
	do{print STDOUT "Connection error: ".$response->status_line,"\r\n";next;} unless $response->is_success;
		my $CurrentUser=$1 if $source=~/.+?::.+?::\s(.*)<\/title>/;
		my $myID=$1 if $source=~/var PlayerId=(\d{1,7});/;
		my $turn=$1 if $source=~/var TurnN=(\d{4});/;
		our $AccType='www' if $source=~/Paid=0;/; # для старой вселенной надо ставить 'free'
		$AccType='www' if $source=~/Paid=1;/;
		print STDOUT "\r\n>>>User logged in us player $CurrentUser (ID=$myID)\n";
  return ($AccType, $source);
}

sub Logoff($){
#logoff
 my $AccType = $_[0];
 print STDOUT "\r\nDo Logoff\r\n";
 $browser->get("http://$AccType".$url_base.'/logoff/');
}

